@@ -1,18 +1,13 @@
 package org.jsoup.helper;
 
-import org.jsoup.Connection;
-import org.jsoup.HttpStatusException;
-import org.jsoup.UnsupportedMimeTypeException;
+import org.jsoup.*;
 import org.jsoup.nodes.Document;
 import org.jsoup.parser.Parser;
 import org.jsoup.parser.TokenQueue;
 
 import javax.net.ssl.*;
 import java.io.*;
-import java.net.HttpURLConnection;
-import java.net.MalformedURLException;
-import java.net.URL;
-import java.net.URLEncoder;
+import java.net.*;
 import java.nio.ByteBuffer;
 import java.nio.charset.Charset;
 import java.nio.charset.IllegalCharsetNameException;
@@ -23,15 +18,25 @@ import java.util.*;
 import java.util.regex.Pattern;
 import java.util.zip.GZIPInputStream;
 
+import static org.jsoup.Connection.Method.HEAD;
+
 /**
  * Implementation of {@link Connection}.
  * @see org.jsoup.Jsoup#connect(String)
  */
 public class HttpConnection implements Connection {
     public static final String  CONTENT_ENCODING = "Content-Encoding";
+    /**
+     * Many users would get caught by not setting a user-agent and therefore getting different responses on their desktop
+     * vs in jsoup, which would otherwise default to {@code Java}. So by default, use a desktop UA.
+     */
+    public static final String DEFAULT_UA =
+        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.143 Safari/537.36";
+    private static final String USER_AGENT = "User-Agent";
     private static final String CONTENT_TYPE = "Content-Type";
     private static final String MULTIPART_FORM_DATA = "multipart/form-data";
     private static final String FORM_URL_ENCODED = "application/x-www-form-urlencoded";
+    private static final int HTTP_TEMP_REDIR = 307; // http/1.1 temporary redirect, not in Java's set.
 
     public static Connection connect(String url) {
         Connection con = new HttpConnection();
@@ -45,12 +50,30 @@ public class HttpConnection implements Connection {
         return con;
     }
 
+    /**
+     * Encodes the input URL into a safe ASCII URL string
+     * @param url unescaped URL
+     * @return escaped URL
+     */
 	private static String encodeUrl(String url) {
-		if(url == null)
-			return null;
-    	return url.replaceAll(" ", "%20");
+        try {
+            URL u = new URL(url);
+            return encodeUrl(u).toExternalForm();
+        } catch (Exception e) {
+            return url;
+        }
 	}
 
+	private static URL encodeUrl(URL u) {
+        try {
+            //  odd way to encode urls, but it works!
+            final URI uri = new URI(u.getProtocol(), u.getUserInfo(), u.getHost(), u.getPort(), u.getPath(), u.getQuery(), u.getRef());
+            return new URL(uri.toASCIIString());
+        } catch (Exception e) {
+            return u;
+        }
+    }
+
     private static String encodeMimeName(String val) {
         if (val == null)
             return null;
@@ -80,9 +103,19 @@ public class HttpConnection implements Connection {
         return this;
     }
 
+    public Connection proxy(Proxy proxy) {
+        req.proxy(proxy);
+        return this;
+    }
+
+    public Connection proxy(String host, int port) {
+        req.proxy(host, port);
+        return this;
+    }
+
     public Connection userAgent(String userAgent) {
         Validate.notNull(userAgent, "User agent must not be null");
-        req.header("User-Agent", userAgent);
+        req.header(USER_AGENT, userAgent);
         return this;
     }
 
@@ -166,11 +199,33 @@ public class HttpConnection implements Connection {
         return this;
     }
 
+    public Connection.KeyVal data(String key) {
+        Validate.notEmpty(key, "Data key must not be empty");
+        for (Connection.KeyVal keyVal : request().data()) {
+            if (keyVal.key().equals(key))
+                return keyVal;
+        }
+        return null;
+    }
+
+    public Connection requestBody(String body) {
+        req.requestBody(body);
+        return this;
+    }
+
     public Connection header(String name, String value) {
         req.header(name, value);
         return this;
     }
 
+    public Connection headers(Map<String,String> headers) {
+        Validate.notNull(headers, "Header map must not be null");
+        for (Map.Entry<String,String> entry : headers.entrySet()) {
+            req.header(entry.getKey(),entry.getValue());
+        }
+        return this;
+    }
+
     public Connection cookie(String name, String value) {
         req.cookie(name, value);
         return this;
@@ -263,7 +318,61 @@ public class HttpConnection implements Connection {
 
         public String header(String name) {
             Validate.notNull(name, "Header name must not be null");
-            return getHeaderCaseInsensitive(name);
+            String val = getHeaderCaseInsensitive(name);
+            if (val != null) {
+                // headers should be ISO8859 - but values are often actually UTF-8. Test if it looks like UTF8 and convert if so
+                val = fixHeaderEncoding(val);
+            }
+            return val;
+        }
+
+        private static String fixHeaderEncoding(String val) {
+            try {
+                byte[] bytes = val.getBytes("ISO-8859-1");
+                if (!looksLikeUtf8(bytes))
+                    return val;
+                return new String(bytes, "UTF-8");
+            } catch (UnsupportedEncodingException e) {
+                // shouldn't happen as these both always exist
+                return val;
+            }
+        }
+
+        private static boolean looksLikeUtf8(byte[] input) {
+            int i = 0;
+            // BOM:
+            if (input.length >= 3 && (input[0] & 0xFF) == 0xEF
+                && (input[1] & 0xFF) == 0xBB & (input[2] & 0xFF) == 0xBF) {
+                i = 3;
+            }
+
+            int end;
+            for (int j = input.length; i < j; ++i) {
+                int o = input[i];
+                if ((o & 0x80) == 0) {
+                    continue; // ASCII
+                }
+
+                // UTF-8 leading:
+                if ((o & 0xE0) == 0xC0) {
+                    end = i + 1;
+                } else if ((o & 0xF0) == 0xE0) {
+                    end = i + 2;
+                } else if ((o & 0xF8) == 0xF0) {
+                    end = i + 3;
+                } else {
+                    return false;
+                }
+
+                while (i < end) {
+                    i++;
+                    o = input[i];
+                    if ((o & 0xC0) != 0x80) {
+                        return false;
+                    }
+                }
+            }
+            return true;
         }
 
         public T header(String name, String value) {
@@ -350,10 +459,12 @@ public class HttpConnection implements Connection {
     }
 
     public static class Request extends HttpConnection.Base<Connection.Request> implements Connection.Request {
+        private Proxy proxy; // nullable
         private int timeoutMilliseconds;
         private int maxBodySizeBytes;
         private boolean followRedirects;
         private Collection<Connection.KeyVal> data;
+        private String body = null;
         private boolean ignoreHttpErrors = false;
         private boolean ignoreContentType = false;
         private Parser parser;
@@ -362,15 +473,30 @@ public class HttpConnection implements Connection {
         private String postDataCharset = DataUtil.defaultCharset;
 
         private Request() {
-            timeoutMilliseconds = 3000;
+            timeoutMilliseconds = 30000; // 30 seconds
             maxBodySizeBytes = 1024 * 1024; // 1MB
             followRedirects = true;
             data = new ArrayList<Connection.KeyVal>();
             method = Method.GET;
             headers.put("Accept-Encoding", "gzip");
+            headers.put(USER_AGENT, DEFAULT_UA);
             parser = Parser.htmlParser();
         }
 
+        public Proxy proxy() {
+            return proxy;
+        }
+
+        public Request proxy(Proxy proxy) {
+            this.proxy = proxy;
+            return this;
+        }
+
+        public Request proxy(String host, int port) {
+            this.proxy = new Proxy(Proxy.Type.HTTP, InetSocketAddress.createUnresolved(host, port));
+            return this;
+        }
+
         public int timeout() {
             return timeoutMilliseconds;
         }
@@ -436,6 +562,15 @@ public class HttpConnection implements Connection {
             return data;
         }
 
+        public Connection.Request requestBody(String body) {
+            this.body = body;
+            return this;
+        }
+
+        public String requestBody() {
+            return body;
+        }
+
         public Request parser(Parser parser) {
             this.parser = parser;
             parserDefined = true;
@@ -498,14 +633,18 @@ public class HttpConnection implements Connection {
             String protocol = req.url().getProtocol();
             if (!protocol.equals("http") && !protocol.equals("https"))
                 throw new MalformedURLException("Only http & https protocols supported");
+            final boolean methodHasBody = req.method().hasBody();
+            final boolean hasRequestBody = req.requestBody() != null;
+            if (!methodHasBody)
+                Validate.isFalse(hasRequestBody, "Cannot set a request body for HTTP method " + req.method());
 
             // set up the request for execution
             String mimeBoundary = null;
-            if (!req.method().hasBody() && req.data().size() > 0) {
-                serialiseRequestUrl(req); // appends query string
-            } else if (req.method().hasBody()) {
+            if (req.data().size() > 0 && (!methodHasBody || hasRequestBody))
+                serialiseRequestUrl(req);
+            else if (methodHasBody)
                 mimeBoundary = setOutputContentType(req);
-            }
+
             HttpURLConnection conn = createConnection(req);
             Response res;
             try {
@@ -520,13 +659,16 @@ public class HttpConnection implements Connection {
 
                 // redirect if there's a location header (from 3xx, or 201 etc)
                 if (res.hasHeader(LOCATION) && req.followRedirects()) {
-                    req.method(Method.GET); // always redirect with a get. any data param from original req are dropped.
-                    req.data().clear();
+                    if (status != HTTP_TEMP_REDIR) {
+                        req.method(Method.GET); // always redirect with a get. any data param from original req are dropped.
+                        req.data().clear();
+                    }
 
                     String location = res.header(LOCATION);
                     if (location != null && location.startsWith("http:/") && location.charAt(6) != '/') // fix broken Location: http:/temp/AAG_New/en/index.php
                         location = location.substring(6);
-                    req.url(StringUtil.resolve(req.url(), encodeUrl(location)));
+                    URL redir = StringUtil.resolve(req.url(), location);
+                    req.url(encodeUrl(redir));
 
                     for (Map.Entry<String, String> cookie : res.cookies.entrySet()) { // add response cookies to request (for e.g. login posts)
                         req.cookie(cookie.getKey(), cookie.getValue());
@@ -555,19 +697,16 @@ public class HttpConnection implements Connection {
                 }
 
                 res.charset = DataUtil.getCharsetFromContentType(res.contentType); // may be null, readInputStream deals with it
-                if (conn.getContentLength() != 0) { // -1 means unknown, chunked. sun throws an IO exception on 500 response with no content when trying to read body
+                if (conn.getContentLength() != 0 && req.method() != HEAD) { // -1 means unknown, chunked. sun throws an IO exception on 500 response with no content when trying to read body
                     InputStream bodyStream = null;
-                    InputStream dataStream = null;
                     try {
-                        dataStream = conn.getErrorStream() != null ? conn.getErrorStream() : conn.getInputStream();
-                        bodyStream = res.hasHeaderWithValue(CONTENT_ENCODING, "gzip") ?
-                                new BufferedInputStream(new GZIPInputStream(dataStream)) :
-                                new BufferedInputStream(dataStream);
+                        bodyStream = conn.getErrorStream() != null ? conn.getErrorStream() : conn.getInputStream();
+                        if (res.hasHeaderWithValue(CONTENT_ENCODING, "gzip"))
+                            bodyStream = new GZIPInputStream(bodyStream);
 
                         res.byteData = DataUtil.readToByteBuffer(bodyStream, req.maxBodySize());
                     } finally {
                         if (bodyStream != null) bodyStream.close();
-                        if (dataStream != null) dataStream.close();
                     }
                 } else {
                     res.byteData = DataUtil.emptyByteBuffer();
@@ -594,6 +733,11 @@ public class HttpConnection implements Connection {
             return charset;
         }
 
+        public Response charset(String charset) {
+            this.charset = charset;
+            return this;
+        }
+
         public String contentType() {
             return contentType;
         }
@@ -625,7 +769,11 @@ public class HttpConnection implements Connection {
 
         // set up connection defaults, and details from request
         private static HttpURLConnection createConnection(Connection.Request req) throws IOException {
-            HttpURLConnection conn = (HttpURLConnection) req.url().openConnection();
+            final HttpURLConnection conn = (HttpURLConnection) (
+                req.proxy() == null ?
+                req.url().openConnection() :
+                req.url().openConnection(req.proxy())
+            );
 
             conn.setRequestMethod(req.method().name());
             conn.setInstanceFollowRedirects(false); // don't rely on native redirection support
@@ -714,7 +862,7 @@ public class HttpConnection implements Connection {
             statusMessage = conn.getResponseMessage();
             contentType = conn.getContentType();
 
-            Map<String, List<String>> resHeaders = conn.getHeaderFields();
+            Map<String, List<String>> resHeaders = createHeaderMap(conn);
             processResponseHeaders(resHeaders);
 
             // if from a redirect, map previous response cookies into this response
@@ -726,6 +874,30 @@ public class HttpConnection implements Connection {
             }
         }
 
+        private static LinkedHashMap<String, List<String>> createHeaderMap(HttpURLConnection conn) {
+            // the default sun impl of conn.getHeaderFields() returns header values out of order
+            final LinkedHashMap<String, List<String>> headers = new LinkedHashMap<String, List<String>>();
+            int i = 0;
+            while (true) {
+                final String key = conn.getHeaderFieldKey(i);
+                final String val = conn.getHeaderField(i);
+                if (key == null && val == null)
+                    break;
+                i++;
+                if (key == null || val == null)
+                    continue; // skip http1.1 line
+
+                if (headers.containsKey(key))
+                    headers.get(key).add(val);
+                else {
+                    final ArrayList<String> vals = new ArrayList<String>();
+                    vals.add(val);
+                    headers.put(key, vals);
+                }
+            }
+            return headers;
+        }
+
         void processResponseHeaders(Map<String, List<String>> resHeaders) {
             for (Map.Entry<String, List<String>> entry : resHeaders.entrySet()) {
                 String name = entry.getKey();
@@ -745,24 +917,30 @@ public class HttpConnection implements Connection {
                         if (cookieName.length() > 0)
                             cookie(cookieName, cookieVal);
                     }
-                } else { // only take the first instance of each header
-                    if (!values.isEmpty())
+                } else { // combine same header names with comma: http://www.w3.org/Protocols/rfc2616/rfc2616-sec4.html#sec4.2
+                    if (values.size() == 1)
                         header(name, values.get(0));
+                    else if (values.size() > 1) {
+                        StringBuilder accum = new StringBuilder();
+                        for (int i = 0; i < values.size(); i++) {
+                            final String val = values.get(i);
+                            if (i != 0)
+                                accum.append(", ");
+                            accum.append(val);
+                        }
+                        header(name, accum.toString());
+                    }
                 }
             }
         }
 
         private static String setOutputContentType(final Connection.Request req) {
-            // multipart mode, for files. add the header if we see something with an inputstream, and return a non-null boundary
-            boolean needsMulti = false;
-            for (Connection.KeyVal keyVal : req.data()) {
-                if (keyVal.hasInputStream()) {
-                    needsMulti = true;
-                    break;
-                }
-            }
             String bound = null;
-            if (needsMulti) {
+            if (req.hasHeader(CONTENT_TYPE)) {
+                // no-op; don't add content type as already set (e.g. for requestBody())
+                // todo - if content type already set, we could add charset or boundary if those aren't included
+            }
+            else if (needsMultipart(req)) {
                 bound = DataUtil.mimeBoundary();
                 req.header(CONTENT_TYPE, MULTIPART_FORM_DATA + "; boundary=" + bound);
             } else {
@@ -773,7 +951,7 @@ public class HttpConnection implements Connection {
 
         private static void writePost(final Connection.Request req, final OutputStream outputStream, final String bound) throws IOException {
             final Collection<Connection.KeyVal> data = req.data();
-            final BufferedWriter w = new BufferedWriter(new OutputStreamWriter(outputStream, DataUtil.defaultCharset));
+            final BufferedWriter w = new BufferedWriter(new OutputStreamWriter(outputStream, req.postDataCharset()));
 
             if (bound != null) {
                 // boundary will be set if we're in multipart mode
@@ -800,7 +978,11 @@ public class HttpConnection implements Connection {
                 w.write("--");
                 w.write(bound);
                 w.write("--");
-            } else {
+            } else if (req.requestBody() != null) {
+                // data will be in query string, we're sending a plaintext body
+                w.write(req.requestBody());
+            }
+            else {
                 // regular form data (application/x-www-form-urlencoded)
                 boolean first = true;
                 for (Connection.KeyVal keyVal : data) {
@@ -848,6 +1030,7 @@ public class HttpConnection implements Connection {
                 first = false;
             }
             for (Connection.KeyVal keyVal : req.data()) {
+                Validate.isFalse(keyVal.hasInputStream(), "InputStream data not supported in URL query string.");
                 if (!first)
                     url.append('&');
                 else
@@ -862,6 +1045,18 @@ public class HttpConnection implements Connection {
         }
     }
 
+    private static boolean needsMultipart(Connection.Request req) {
+        // multipart mode, for files. add the header if we see something with an inputstream, and return a non-null boundary
+        boolean needsMulti = false;
+        for (Connection.KeyVal keyVal : req.data()) {
+            if (keyVal.hasInputStream()) {
+                needsMulti = true;
+                break;
+            }
+        }
+        return needsMulti;
+    }
+
     public static class KeyVal implements Connection.KeyVal {
         private String key;
         private String value;
