@@ -24,14 +24,14 @@ public class Attribute implements Map.Entry<String, String>, Cloneable  {
 
     /**
      * Create a new attribute from unencoded (raw) key and value.
-     * @param key attribute key
+     * @param key attribute key; case is preserved.
      * @param value attribute value
      * @see #createFromEncoded
      */
     public Attribute(String key, String value) {
         Validate.notEmpty(key);
         Validate.notNull(value);
-        this.key = key.trim().toLowerCase();
+        this.key = key.trim();
         this.value = value;
     }
 
@@ -44,12 +44,12 @@ public class Attribute implements Map.Entry<String, String>, Cloneable  {
     }
 
     /**
-     Set the attribute key. Gets normalised as per the constructor method.
+     Set the attribute key; case is preserved.
      @param key the new key; must not be null
      */
     public void setKey(String key) {
         Validate.notEmpty(key);
-        this.key = key.trim().toLowerCase();
+        this.key = key.trim();
     }
 
     /**
@@ -122,7 +122,7 @@ public class Attribute implements Map.Entry<String, String>, Cloneable  {
     /**
      * Collapsible if it's a boolean attribute and value is empty or same as name
      * 
-     * @param out Outputsettings
+     * @param out output settings
      * @return  Returns whether collapsible or not
      */
     protected final boolean shouldCollapseAttribute(Document.OutputSettings out) {
