@@ -2,11 +2,26 @@ package org.jsoup.nodes;
 
 import org.jsoup.helper.StringUtil;
 import org.jsoup.helper.Validate;
+import org.jsoup.parser.ParseSettings;
 import org.jsoup.parser.Parser;
 import org.jsoup.parser.Tag;
-import org.jsoup.select.*;
+import org.jsoup.select.Collector;
+import org.jsoup.select.Elements;
+import org.jsoup.select.Evaluator;
+import org.jsoup.select.NodeTraversor;
+import org.jsoup.select.NodeVisitor;
+import org.jsoup.select.QueryParser;
+import org.jsoup.select.Selector;
 
-import java.util.*;
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
 
@@ -23,6 +38,14 @@ public class Element extends Node {
 
     private static final Pattern classSplit = Pattern.compile("\\s+");
 
+    /**
+     * Create a new, standalone element.
+     * @param tag tag name
+     */
+    public Element(String tag) {
+        this(Tag.valueOf(tag), "", new Attributes());
+    }
+
     /**
      * Create a new, standalone Element. (Standalone in that is has no parent.)
      * 
@@ -45,7 +68,7 @@ public class Element extends Node {
      * @param tag element tag
      * @param baseUri the base URI of this element. It is acceptable for the base URI to be an empty
      *            string, but not null.
-     * @see Tag#valueOf(String)
+     * @see Tag#valueOf(String, ParseSettings)
      */
     public Element(Tag tag, String baseUri) {
         this(tag, baseUri, new Attributes());
@@ -74,7 +97,7 @@ public class Element extends Node {
      */
     public Element tagName(String tagName) {
         Validate.notEmpty(tagName, "Tag name must not be empty.");
-        tag = Tag.valueOf(tagName);
+        tag = Tag.valueOf(tagName, ParseSettings.preserveCase); // preserve the requested tag case
         return this;
     }
 
@@ -103,7 +126,7 @@ public class Element extends Node {
      * @return The id attribute, if present, or an empty string if not.
      */
     public String id() {
-        return attributes.get("id");
+        return attributes.getIgnoreCase("id");
     }
 
     /**
@@ -272,6 +295,24 @@ public class Element extends Node {
     public Elements select(String cssQuery) {
         return Selector.select(cssQuery, this);
     }
+
+    /**
+     * Check if this element matches the given {@link Selector} CSS query.
+     * @param cssQuery a {@link Selector} CSS query
+     * @return if this element matches the query
+     */
+    public boolean is(String cssQuery) {
+        return is(QueryParser.parse(cssQuery));
+    }
+
+    /**
+     * Check if this element matches the given evaluator.
+     * @param evaluator an element evaluator
+     * @return if this element matches
+     */
+    public boolean is(Evaluator evaluator) {
+        return evaluator.matches((Element)this.root(), this);
+    }
     
     /**
      * Add a node child node to this element.
@@ -358,6 +399,7 @@ public class Element extends Node {
      * @return this element
      */
     public Element appendText(String text) {
+        Validate.notNull(text);
         TextNode node = new TextNode(text, baseUri());
         appendChild(node);
         return this;
@@ -370,6 +412,7 @@ public class Element extends Node {
      * @return this element
      */
     public Element prependText(String text) {
+        Validate.notNull(text);
         TextNode node = new TextNode(text, baseUri());
         prependChild(node);
         return this;
@@ -483,7 +526,9 @@ public class Element extends Node {
         if (id().length() > 0)
             return "#" + id();
 
-        StringBuilder selector = new StringBuilder(tagName());
+        // Translate HTML namespace ns:tag to CSS namespace syntax ns|tag
+        String tagName = tagName().replace(':', '|');
+        StringBuilder selector = new StringBuilder(tagName);
         String classes = StringUtil.join(classNames(), ".");
         if (classes.length() > 0)
             selector.append('.').append(classes);
@@ -651,7 +696,7 @@ public class Element extends Node {
      */
     public Elements getElementsByAttribute(String key) {
         Validate.notEmpty(key);
-        key = key.trim().toLowerCase();
+        key = key.trim();
 
         return Collector.collect(new Evaluator.Attribute(key), this);
     }
@@ -664,7 +709,7 @@ public class Element extends Node {
      */
     public Elements getElementsByAttributeStarting(String keyPrefix) {
         Validate.notEmpty(keyPrefix);
-        keyPrefix = keyPrefix.trim().toLowerCase();
+        keyPrefix = keyPrefix.trim();
 
         return Collector.collect(new Evaluator.AttributeStarting(keyPrefix), this);
     }
@@ -991,6 +1036,9 @@ public class Element extends Node {
             if (childNode instanceof DataNode) {
                 DataNode data = (DataNode) childNode;
                 sb.append(data.getWholeData());
+            } else if (childNode instanceof Comment) {
+                Comment comment = (Comment) childNode;
+                sb.append(comment.getData());
             } else if (childNode instanceof Element) {
                 Element element = (Element) childNode;
                 String elementData = element.data();
@@ -1039,23 +1087,45 @@ public class Element extends Node {
      * @param className name of class to check for
      * @return true if it does, false if not
      */
-    /*
-    Used by common .class selector, so perf tweaked to reduce object creation vs hitting classnames().
-
-    Wiki: 71, 13 (5.4x)
-    CNN: 227, 91 (2.5x)
-    Alterslash: 59, 4 (14.8x)
-    Jsoup: 14, 1 (14x)
-    */
+    // performance sensitive
     public boolean hasClass(String className) {
-        String classAttr = attributes.get("class");
-        if (classAttr.equals("") || classAttr.length() < className.length())
-            return false;
+        final String classAttr = attributes.get("class");
+        final int len = classAttr.length();
+        final int wantLen = className.length();
 
-        final String[] classes = classSplit.split(classAttr);
-        for (String name : classes) {
-            if (className.equalsIgnoreCase(name))
-                return true;
+        if (len == 0 || len < wantLen) {
+            return false;
+        }
+
+        // if both lengths are equal, only need compare the className with the attribute
+        if (len == wantLen) {
+            return className.equalsIgnoreCase(classAttr);
+        }
+
+        // otherwise, scan for whitespace and compare regions (with no string or arraylist allocations)
+        boolean inClass = false;
+        int start = 0;
+        for (int i = 0; i < len; i++) {
+            if (Character.isWhitespace(classAttr.charAt(i))) {
+                if (inClass) {
+                    // white space ends a class name, compare it with the requested one, ignore case
+                    if (i - start == wantLen && classAttr.regionMatches(true, start, className, 0, wantLen)) {
+                        return true;
+                    }
+                    inClass = false;
+                }
+            } else {
+                if (!inClass) {
+                    // we're in a class name : keep the start of the substring
+                    inClass = true;
+                    start = i;
+                }
+            }
+        }
+
+        // check the last entry
+        if (inClass && len - start == wantLen) {
+            return classAttr.regionMatches(true, start, className, 0, wantLen);
         }
 
         return false;
@@ -1133,9 +1203,15 @@ public class Element extends Node {
         return this;
     }
 
-    void outerHtmlHead(StringBuilder accum, int depth, Document.OutputSettings out) {
-        if (accum.length() > 0 && out.prettyPrint() && (tag.formatAsBlock() || (parent() != null && parent().tag().formatAsBlock()) || out.outline()) )
-            indent(accum, depth, out);
+    void outerHtmlHead(Appendable accum, int depth, Document.OutputSettings out) throws IOException {
+        if (out.prettyPrint() && (tag.formatAsBlock() || (parent() != null && parent().tag().formatAsBlock()) || out.outline())) {
+            if (accum instanceof StringBuilder) {
+                if (((StringBuilder) accum).length() > 0)
+                    indent(accum, depth, out);
+            } else {
+                indent(accum, depth, out);
+            }
+        }
         accum
                 .append("<")
                 .append(tagName());
@@ -1152,7 +1228,7 @@ public class Element extends Node {
             accum.append(">");
     }
 
-    void outerHtmlTail(StringBuilder accum, int depth, Document.OutputSettings out) {
+	void outerHtmlTail(Appendable accum, int depth, Document.OutputSettings out) throws IOException {
         if (!(childNodes.isEmpty() && tag.isSelfClosing())) {
             if (out.prettyPrint() && (!childNodes.isEmpty() && (
                     tag.formatAsBlock() || (out.outline() && (childNodes.size()>1 || (childNodes.size()==1 && !(childNodes.get(0) instanceof TextNode))))
@@ -1179,6 +1255,17 @@ public class Element extends Node {
         for (Node node : childNodes)
             node.outerHtml(accum);
     }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public <T extends Appendable> T html(T appendable) {
+        for (Node node : childNodes)
+            node.outerHtml(appendable);
+
+        return appendable;
+    }
     
     /**
      * Set this element's inner HTML. Clears the existing HTML first.
@@ -1192,28 +1279,10 @@ public class Element extends Node {
         return this;
     }
 
-    public String toString() {
+	public String toString() {
         return outerHtml();
     }
 
-    @Override
-    public boolean equals(Object o) {
-        if (this == o) return true;
-        if (o == null || getClass() != o.getClass()) return false;
-        if (!super.equals(o)) return false;
-
-        Element element = (Element) o;
-
-        return tag.equals(element.tag);
-    }
-
-    @Override
-    public int hashCode() {
-        int result = super.hashCode();
-        result = 31 * result + (tag != null ? tag.hashCode() : 0);
-        return result;
-    }
-
     @Override
     public Element clone() {
         return (Element) super.clone();
