@@ -19,7 +19,8 @@ enum HtmlTreeBuilderState {
                 // todo: parse error check on expected doctypes
                 // todo: quirk state check on doctype ids
                 Token.Doctype d = t.asDoctype();
-                DocumentType doctype = new DocumentType(d.getName(), d.getPublicIdentifier(), d.getSystemIdentifier(), tb.getBaseUri());
+                DocumentType doctype = new DocumentType(
+                    tb.settings.normalizeTag(d.getName()), d.getPubSysKey(), d.getPublicIdentifier(), d.getSystemIdentifier(), tb.getBaseUri());
                 tb.getDocument().appendChild(doctype);
                 if (d.isForceQuirks())
                     tb.getDocument().quirksMode(Document.QuirksMode.quirks);
@@ -41,10 +42,10 @@ enum HtmlTreeBuilderState {
                 tb.insert(t.asComment());
             } else if (isWhitespace(t)) {
                 return true; // ignore whitespace
-            } else if (t.isStartTag() && t.asStartTag().name().equals("html")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("html")) {
                 tb.insert(t.asStartTag());
                 tb.transition(BeforeHead);
-            } else if (t.isEndTag() && (StringUtil.in(t.asEndTag().name(), "head", "body", "html", "br"))) {
+            } else if (t.isEndTag() && (StringUtil.in(t.asEndTag().normalName(), "head", "body", "html", "br"))) {
                 return anythingElse(t, tb);
             } else if (t.isEndTag()) {
                 tb.error(this);
@@ -70,13 +71,13 @@ enum HtmlTreeBuilderState {
             } else if (t.isDoctype()) {
                 tb.error(this);
                 return false;
-            } else if (t.isStartTag() && t.asStartTag().name().equals("html")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("html")) {
                 return InBody.process(t, tb); // does not transition
-            } else if (t.isStartTag() && t.asStartTag().name().equals("head")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("head")) {
                 Element head = tb.insert(t.asStartTag());
                 tb.setHeadElement(head);
                 tb.transition(InHead);
-            } else if (t.isEndTag() && (StringUtil.in(t.asEndTag().name(), "head", "body", "html", "br"))) {
+            } else if (t.isEndTag() && (StringUtil.in(t.asEndTag().normalName(), "head", "body", "html", "br"))) {
                 tb.processStartTag("head");
                 return tb.process(t);
             } else if (t.isEndTag()) {
@@ -104,7 +105,7 @@ enum HtmlTreeBuilderState {
                     return false;
                 case StartTag:
                     Token.StartTag start = t.asStartTag();
-                    String name = start.name();
+                    String name = start.normalName();
                     if (name.equals("html")) {
                         return InBody.process(t, tb);
                     } else if (StringUtil.in(name, "base", "basefont", "bgsound", "command", "link")) {
@@ -139,7 +140,7 @@ enum HtmlTreeBuilderState {
                     break;
                 case EndTag:
                     Token.EndTag end = t.asEndTag();
-                    name = end.name();
+                    name = end.normalName();
                     if (name.equals("head")) {
                         tb.pop();
                         tb.transition(AfterHead);
@@ -165,17 +166,17 @@ enum HtmlTreeBuilderState {
         boolean process(Token t, HtmlTreeBuilder tb) {
             if (t.isDoctype()) {
                 tb.error(this);
-            } else if (t.isStartTag() && t.asStartTag().name().equals("html")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("html")) {
                 return tb.process(t, InBody);
-            } else if (t.isEndTag() && t.asEndTag().name().equals("noscript")) {
+            } else if (t.isEndTag() && t.asEndTag().normalName().equals("noscript")) {
                 tb.pop();
                 tb.transition(InHead);
-            } else if (isWhitespace(t) || t.isComment() || (t.isStartTag() && StringUtil.in(t.asStartTag().name(),
+            } else if (isWhitespace(t) || t.isComment() || (t.isStartTag() && StringUtil.in(t.asStartTag().normalName(),
                     "basefont", "bgsound", "link", "meta", "noframes", "style"))) {
                 return tb.process(t, InHead);
-            } else if (t.isEndTag() && t.asEndTag().name().equals("br")) {
+            } else if (t.isEndTag() && t.asEndTag().normalName().equals("br")) {
                 return anythingElse(t, tb);
-            } else if ((t.isStartTag() && StringUtil.in(t.asStartTag().name(), "head", "noscript")) || t.isEndTag()) {
+            } else if ((t.isStartTag() && StringUtil.in(t.asStartTag().normalName(), "head", "noscript")) || t.isEndTag()) {
                 tb.error(this);
                 return false;
             } else {
@@ -200,7 +201,7 @@ enum HtmlTreeBuilderState {
                 tb.error(this);
             } else if (t.isStartTag()) {
                 Token.StartTag startTag = t.asStartTag();
-                String name = startTag.name();
+                String name = startTag.normalName();
                 if (name.equals("html")) {
                     return tb.process(t, InBody);
                 } else if (name.equals("body")) {
@@ -223,7 +224,7 @@ enum HtmlTreeBuilderState {
                     anythingElse(t, tb);
                 }
             } else if (t.isEndTag()) {
-                if (StringUtil.in(t.asEndTag().name(), "body", "html")) {
+                if (StringUtil.in(t.asEndTag().normalName(), "body", "html")) {
                     anythingElse(t, tb);
                 } else {
                     tb.error(this);
@@ -270,7 +271,7 @@ enum HtmlTreeBuilderState {
                 }
                 case StartTag:
                     Token.StartTag startTag = t.asStartTag();
-                    String name = startTag.name();
+                    String name = startTag.normalName();
                     if (name.equals("a")) {
                         if (tb.getActiveFormattingElement("a") != null) {
                             tb.error(this);
@@ -555,7 +556,7 @@ enum HtmlTreeBuilderState {
 
                 case EndTag:
                     Token.EndTag endTag = t.asEndTag();
-                    name = endTag.name();
+                    name = endTag.normalName();
                     if (StringUtil.inSorted(name, Constants.InBodyEndAdoptionFormatters)) {
                         // Adoption Agency Algorithm.
                         for (int i = 0; i < 8; i++) {
@@ -608,7 +609,8 @@ enum HtmlTreeBuilderState {
                                 } else if (node == formatEl)
                                     break;
 
-                                Element replacement = new Element(Tag.valueOf(node.nodeName()), tb.getBaseUri());
+                                Element replacement = new Element(Tag.valueOf(node.nodeName(), ParseSettings.preserveCase), tb.getBaseUri());
+                                // case will follow the original node (so honours ParseSettings)
                                 tb.replaceActiveFormattingElement(node, replacement);
                                 tb.replaceOnStack(node, replacement);
                                 node = replacement;
@@ -759,7 +761,7 @@ enum HtmlTreeBuilderState {
         }
 
         boolean anyOtherEndTag(Token t, HtmlTreeBuilder tb) {
-            String name = t.asEndTag().name();
+            String name = t.asEndTag().normalName();
             ArrayList<Element> stack = tb.getStack();
             for (int pos = stack.size() -1; pos >= 0; pos--) {
                 Element node = stack.get(pos);
@@ -813,7 +815,7 @@ enum HtmlTreeBuilderState {
                 return false;
             } else if (t.isStartTag()) {
                 Token.StartTag startTag = t.asStartTag();
-                String name = startTag.name();
+                String name = startTag.normalName();
                 if (name.equals("caption")) {
                     tb.clearStackToTableContext();
                     tb.insertMarkerToFormattingElements();
@@ -859,7 +861,7 @@ enum HtmlTreeBuilderState {
                 return true; // todo: check if should return processed http://www.whatwg.org/specs/web-apps/current-work/multipage/tree-construction.html#parsing-main-intable
             } else if (t.isEndTag()) {
                 Token.EndTag endTag = t.asEndTag();
-                String name = endTag.name();
+                String name = endTag.normalName();
 
                 if (name.equals("table")) {
                     if (!tb.inTableScope(name)) {
@@ -937,9 +939,9 @@ enum HtmlTreeBuilderState {
     },
     InCaption {
         boolean process(Token t, HtmlTreeBuilder tb) {
-            if (t.isEndTag() && t.asEndTag().name().equals("caption")) {
+            if (t.isEndTag() && t.asEndTag().normalName().equals("caption")) {
                 Token.EndTag endTag = t.asEndTag();
-                String name = endTag.name();
+                String name = endTag.normalName();
                 if (!tb.inTableScope(name)) {
                     tb.error(this);
                     return false;
@@ -952,15 +954,15 @@ enum HtmlTreeBuilderState {
                     tb.transition(InTable);
                 }
             } else if ((
-                    t.isStartTag() && StringUtil.in(t.asStartTag().name(),
+                    t.isStartTag() && StringUtil.in(t.asStartTag().normalName(),
                             "caption", "col", "colgroup", "tbody", "td", "tfoot", "th", "thead", "tr") ||
-                            t.isEndTag() && t.asEndTag().name().equals("table"))
+                            t.isEndTag() && t.asEndTag().normalName().equals("table"))
                     ) {
                 tb.error(this);
                 boolean processed = tb.processEndTag("caption");
                 if (processed)
                     return tb.process(t);
-            } else if (t.isEndTag() && StringUtil.in(t.asEndTag().name(),
+            } else if (t.isEndTag() && StringUtil.in(t.asEndTag().normalName(),
                     "body", "col", "colgroup", "html", "tbody", "td", "tfoot", "th", "thead", "tr")) {
                 tb.error(this);
                 return false;
@@ -985,7 +987,7 @@ enum HtmlTreeBuilderState {
                     break;
                 case StartTag:
                     Token.StartTag startTag = t.asStartTag();
-                    String name = startTag.name();
+                    String name = startTag.normalName();
                     if (name.equals("html"))
                         return tb.process(t, InBody);
                     else if (name.equals("col"))
@@ -995,7 +997,7 @@ enum HtmlTreeBuilderState {
                     break;
                 case EndTag:
                     Token.EndTag endTag = t.asEndTag();
-                    name = endTag.name();
+                    name = endTag.normalName();
                     if (name.equals("colgroup")) {
                         if (tb.currentElement().nodeName().equals("html")) { // frag case
                             tb.error(this);
@@ -1030,7 +1032,7 @@ enum HtmlTreeBuilderState {
             switch (t.type) {
                 case StartTag:
                     Token.StartTag startTag = t.asStartTag();
-                    String name = startTag.name();
+                    String name = startTag.normalName();
                     if (name.equals("tr")) {
                         tb.clearStackToTableBodyContext();
                         tb.insert(startTag);
@@ -1046,7 +1048,7 @@ enum HtmlTreeBuilderState {
                     break;
                 case EndTag:
                     Token.EndTag endTag = t.asEndTag();
-                    name = endTag.name();
+                    name = endTag.normalName();
                     if (StringUtil.in(name, "tbody", "tfoot", "thead")) {
                         if (!tb.inTableScope(name)) {
                             tb.error(this);
@@ -1089,7 +1091,7 @@ enum HtmlTreeBuilderState {
         boolean process(Token t, HtmlTreeBuilder tb) {
             if (t.isStartTag()) {
                 Token.StartTag startTag = t.asStartTag();
-                String name = startTag.name();
+                String name = startTag.normalName();
 
                 if (StringUtil.in(name, "th", "td")) {
                     tb.clearStackToTableRowContext();
@@ -1103,7 +1105,7 @@ enum HtmlTreeBuilderState {
                 }
             } else if (t.isEndTag()) {
                 Token.EndTag endTag = t.asEndTag();
-                String name = endTag.name();
+                String name = endTag.normalName();
 
                 if (name.equals("tr")) {
                     if (!tb.inTableScope(name)) {
@@ -1150,7 +1152,7 @@ enum HtmlTreeBuilderState {
         boolean process(Token t, HtmlTreeBuilder tb) {
             if (t.isEndTag()) {
                 Token.EndTag endTag = t.asEndTag();
-                String name = endTag.name();
+                String name = endTag.normalName();
 
                 if (StringUtil.in(name, "td", "th")) {
                     if (!tb.inTableScope(name)) {
@@ -1178,7 +1180,7 @@ enum HtmlTreeBuilderState {
                     return anythingElse(t, tb);
                 }
             } else if (t.isStartTag() &&
-                    StringUtil.in(t.asStartTag().name(),
+                    StringUtil.in(t.asStartTag().normalName(),
                             "caption", "col", "colgroup", "tbody", "td", "tfoot", "th", "thead", "tr")) {
                 if (!(tb.inTableScope("td") || tb.inTableScope("th"))) {
                     tb.error(this);
@@ -1223,7 +1225,7 @@ enum HtmlTreeBuilderState {
                     return false;
                 case StartTag:
                     Token.StartTag start = t.asStartTag();
-                    String name = start.name();
+                    String name = start.normalName();
                     if (name.equals("html"))
                         return tb.process(start, InBody);
                     else if (name.equals("option")) {
@@ -1252,7 +1254,7 @@ enum HtmlTreeBuilderState {
                     break;
                 case EndTag:
                     Token.EndTag end = t.asEndTag();
-                    name = end.name();
+                    name = end.normalName();
                     if (name.equals("optgroup")) {
                         if (tb.currentElement().nodeName().equals("option") && tb.aboveOnStack(tb.currentElement()) != null && tb.aboveOnStack(tb.currentElement()).nodeName().equals("optgroup"))
                             tb.processEndTag("option");
@@ -1293,13 +1295,13 @@ enum HtmlTreeBuilderState {
     },
     InSelectInTable {
         boolean process(Token t, HtmlTreeBuilder tb) {
-            if (t.isStartTag() && StringUtil.in(t.asStartTag().name(), "caption", "table", "tbody", "tfoot", "thead", "tr", "td", "th")) {
+            if (t.isStartTag() && StringUtil.in(t.asStartTag().normalName(), "caption", "table", "tbody", "tfoot", "thead", "tr", "td", "th")) {
                 tb.error(this);
                 tb.processEndTag("select");
                 return tb.process(t);
-            } else if (t.isEndTag() && StringUtil.in(t.asEndTag().name(), "caption", "table", "tbody", "tfoot", "thead", "tr", "td", "th")) {
+            } else if (t.isEndTag() && StringUtil.in(t.asEndTag().normalName(), "caption", "table", "tbody", "tfoot", "thead", "tr", "td", "th")) {
                 tb.error(this);
-                if (tb.inTableScope(t.asEndTag().name())) {
+                if (tb.inTableScope(t.asEndTag().normalName())) {
                     tb.processEndTag("select");
                     return (tb.process(t));
                 } else
@@ -1318,9 +1320,9 @@ enum HtmlTreeBuilderState {
             } else if (t.isDoctype()) {
                 tb.error(this);
                 return false;
-            } else if (t.isStartTag() && t.asStartTag().name().equals("html")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("html")) {
                 return tb.process(t, InBody);
-            } else if (t.isEndTag() && t.asEndTag().name().equals("html")) {
+            } else if (t.isEndTag() && t.asEndTag().normalName().equals("html")) {
                 if (tb.isFragmentParsing()) {
                     tb.error(this);
                     return false;
@@ -1348,7 +1350,7 @@ enum HtmlTreeBuilderState {
                 return false;
             } else if (t.isStartTag()) {
                 Token.StartTag start = t.asStartTag();
-                String name = start.name();
+                String name = start.normalName();
                 if (name.equals("html")) {
                     return tb.process(start, InBody);
                 } else if (name.equals("frameset")) {
@@ -1361,7 +1363,7 @@ enum HtmlTreeBuilderState {
                     tb.error(this);
                     return false;
                 }
-            } else if (t.isEndTag() && t.asEndTag().name().equals("frameset")) {
+            } else if (t.isEndTag() && t.asEndTag().normalName().equals("frameset")) {
                 if (tb.currentElement().nodeName().equals("html")) { // frag
                     tb.error(this);
                     return false;
@@ -1392,11 +1394,11 @@ enum HtmlTreeBuilderState {
             } else if (t.isDoctype()) {
                 tb.error(this);
                 return false;
-            } else if (t.isStartTag() && t.asStartTag().name().equals("html")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("html")) {
                 return tb.process(t, InBody);
-            } else if (t.isEndTag() && t.asEndTag().name().equals("html")) {
+            } else if (t.isEndTag() && t.asEndTag().normalName().equals("html")) {
                 tb.transition(AfterAfterFrameset);
-            } else if (t.isStartTag() && t.asStartTag().name().equals("noframes")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("noframes")) {
                 return tb.process(t, InHead);
             } else if (t.isEOF()) {
                 // cool your heels, we're complete
@@ -1411,7 +1413,7 @@ enum HtmlTreeBuilderState {
         boolean process(Token t, HtmlTreeBuilder tb) {
             if (t.isComment()) {
                 tb.insert(t.asComment());
-            } else if (t.isDoctype() || isWhitespace(t) || (t.isStartTag() && t.asStartTag().name().equals("html"))) {
+            } else if (t.isDoctype() || isWhitespace(t) || (t.isStartTag() && t.asStartTag().normalName().equals("html"))) {
                 return tb.process(t, InBody);
             } else if (t.isEOF()) {
                 // nice work chuck
@@ -1427,11 +1429,11 @@ enum HtmlTreeBuilderState {
         boolean process(Token t, HtmlTreeBuilder tb) {
             if (t.isComment()) {
                 tb.insert(t.asComment());
-            } else if (t.isDoctype() || isWhitespace(t) || (t.isStartTag() && t.asStartTag().name().equals("html"))) {
+            } else if (t.isDoctype() || isWhitespace(t) || (t.isStartTag() && t.asStartTag().normalName().equals("html"))) {
                 return tb.process(t, InBody);
             } else if (t.isEOF()) {
                 // nice work chuck
-            } else if (t.isStartTag() && t.asStartTag().name().equals("noframes")) {
+            } else if (t.isStartTag() && t.asStartTag().normalName().equals("noframes")) {
                 return tb.process(t, InHead);
             } else {
                 tb.error(this);
