@@ -25,7 +25,7 @@ public class Tag {
     private boolean formSubmit = false; // a control that can be submitted in a form: input etc
 
     private Tag(String tagName) {
-        this.tagName = tagName.toLowerCase();
+        this.tagName = tagName;
     }
 
     /**
@@ -39,18 +39,20 @@ public class Tag {
 
     /**
      * Get a Tag by name. If not previously defined (unknown), returns a new generic tag, that can do anything.
-     * <p/>
+     * <p>
      * Pre-defined tags (P, DIV etc) will be ==, but unknown tags are not registered and will only .equals().
-     *
+     * </p>
+     * 
      * @param tagName Name of tag, e.g. "p". Case insensitive.
+     * @param settings used to control tag name sensitivity
      * @return The tag, either defined or new generic.
      */
-    public static Tag valueOf(String tagName) {
+    public static Tag valueOf(String tagName, ParseSettings settings) {
         Validate.notNull(tagName);
         Tag tag = tags.get(tagName);
 
         if (tag == null) {
-            tagName = tagName.trim().toLowerCase();
+            tagName = settings.normalizeTag(tagName);
             Validate.notEmpty(tagName);
             tag = tags.get(tagName);
 
@@ -64,6 +66,19 @@ public class Tag {
         return tag;
     }
 
+    /**
+     * Get a Tag by name. If not previously defined (unknown), returns a new generic tag, that can do anything.
+     * <p>
+     * Pre-defined tags (P, DIV etc) will be ==, but unknown tags are not registered and will only .equals().
+     * </p>
+     *
+     * @param tagName Name of tag, e.g. "p". <b>Case sensitive</b>.
+     * @return The tag, either defined or new generic.
+     */
+    public static Tag valueOf(String tagName) {
+        return valueOf(tagName, ParseSettings.preserveCase);
+    }
+
     /**
      * Gets if this is a block tag.
      *
@@ -149,7 +164,7 @@ public class Tag {
     /**
      * Get if this tag should preserve whitespace within child text nodes.
      *
-     * @return if preserve whitepace
+     * @return if preserve whitespace
      */
     public boolean preserveWhitespace() {
         return preserveWhitespace;
@@ -183,6 +198,7 @@ public class Tag {
 
         Tag tag = (Tag) o;
 
+        if (!tagName.equals(tag.tagName)) return false;
         if (canContainBlock != tag.canContainBlock) return false;
         if (canContainInline != tag.canContainInline) return false;
         if (empty != tag.empty) return false;
@@ -191,10 +207,7 @@ public class Tag {
         if (preserveWhitespace != tag.preserveWhitespace) return false;
         if (selfClosing != tag.selfClosing) return false;
         if (formList != tag.formList) return false;
-        if (formSubmit != tag.formSubmit) return false;
-        if (!tagName.equals(tag.tagName)) return false;
-
-        return true;
+        return formSubmit == tag.formSubmit;
     }
 
     @Override
@@ -212,6 +225,7 @@ public class Tag {
         return result;
     }
 
+    @Override
     public String toString() {
         return tagName;
     }
@@ -223,14 +237,16 @@ public class Tag {
             "noframes", "section", "nav", "aside", "hgroup", "header", "footer", "p", "h1", "h2", "h3", "h4", "h5", "h6",
             "ul", "ol", "pre", "div", "blockquote", "hr", "address", "figure", "figcaption", "form", "fieldset", "ins",
             "del", "s", "dl", "dt", "dd", "li", "table", "caption", "thead", "tfoot", "tbody", "colgroup", "col", "tr", "th",
-            "td", "video", "audio", "canvas", "details", "menu", "plaintext"
+            "td", "video", "audio", "canvas", "details", "menu", "plaintext", "template", "article", "main",
+            "svg", "math"
     };
     private static final String[] inlineTags = {
             "object", "base", "font", "tt", "i", "b", "u", "big", "small", "em", "strong", "dfn", "code", "samp", "kbd",
             "var", "cite", "abbr", "time", "acronym", "mark", "ruby", "rt", "rp", "a", "img", "br", "wbr", "map", "q",
             "sub", "sup", "bdo", "iframe", "embed", "span", "input", "select", "textarea", "label", "button", "optgroup",
             "option", "legend", "datalist", "keygen", "output", "progress", "meter", "area", "param", "source", "track",
-            "summary", "command", "device", "area", "basefont", "bgsound", "menuitem", "param", "source", "track"
+            "summary", "command", "device", "area", "basefont", "bgsound", "menuitem", "param", "source", "track",
+            "data", "bdi"
     };
     private static final String[] emptyTags = {
             "meta", "link", "base", "frame", "img", "br", "wbr", "embed", "hr", "input", "keygen", "col", "command",
